function factory(obj){
    if(obj && obj.method){
        if(obj.data)
            RESPY_PageParser[obj.method](obj.data);
        else
            RESPY_PageParser[obj.method]();
    }
}

let RESPY_PageParser = {
    init : function () {
        chrome.runtime.onConnect.addListener(function(port){
            port.onMessage.addListener(factory);
        });
        this.onExtensionInit();
    },

    // Вызывается через factory
    allowParse : function (params) {
        let data = {
            allow : false,
            allowExtractPhone : false,
            pageData : {}
        };

        data.tab_id = params.tab_id;
        switch (true) {
            case /youla.ru/.test(window.location.host) : {
                data.pageData = RESPY_YoulaParser.getData();
                break;
            }
            case /www.avito.ru/.test(window.location.host) : {
                data.pageData = RESPY_AvitoParser.getData();
                break;
            }
            case /cian.ru/.test(window.location.host) : {
                data.pageData = RESPY_CianParser.getData();
                break;
            }
            case /realty.yandex.ru/.test(window.location.host) : {
                data.pageData = RESPY_YandexParser.getData();
                break;
            }
            case /realty.ya.ru/.test(window.location.host) : {
                data.pageData = RESPY_YandexParser.getData();
                break;
            }
            case /domclick.ru/.test(window.location.host) : {
                data.pageData = RESPY_DomclickParser.getData();
                break;
            }
            default : {
                data.pageData = {native_id :null, source: null, region : null};
                break;
            }
        }

        if(data.pageData.native_id && data.pageData.source && (data.pageData.region || data.pageData.address)) {
            data.pageData.url = window.location.href;
            data.allow = true;
        }

        if(data.pageData.allowExtractPhone){
            data.allowExtractPhone = true;
        }

        let port = chrome.runtime.connect();
        port.postMessage({method:'setAllowParseData', data : data});
        if( data.pageData.hasOwnProperty('allowExtractPhone') && data.pageData.allowExtractPhone) {
            let extractPhoneButton = document.createElement("button");
            extractPhoneButton.classList.add('crm-rebpm-extract-phone-button');
            extractPhoneButton.appendChild(document.createTextNode("Получить телефон"));
            $('body').append(extractPhoneButton);
            $('body').on('click', '.crm-rebpm-extract-phone-button',function (event) {
                chrome.runtime.sendMessage({from : 'content', subject : 'extractPhone', data : data.pageData}).then((response) => {
                    if(response.hasOwnProperty('commands') && response.commands) {
                        response.commands.forEach(function(item, i){
                            eval(item);
                        });
                        let after_commands = [];
                        if(response.hasOwnProperty('after_commands') && response.after_commands) {
                            after_commands = response.after_commands;
                        }
                        setTimeout(function(){
                            if(response.hasOwnProperty('selector') && response.selector) {
                                let resultSelector  = $(response.selector);
                                if(resultSelector.length > 0){
                                    let phoneParse = resultSelector.html();
                                    let extractPhoneData = data.pageData;
                                    extractPhoneData['phone'] = phoneParse;
                                    extractPhoneData['salt'] = response.salt;
                                    chrome.runtime.sendMessage({
                                        from : 'content',
                                        subject : 'extractPhoneSave',
                                        data :extractPhoneData
                                    }).then((response) => {


                                        if(response.hasOwnProperty('save') && response.save) {
                                            alert('телефон успешно добавлен');
                                        }
                                        if(response.hasOwnProperty('error') && response.error) {
                                            alert(response.error);
                                        }
                                        after_commands.forEach(function(item, i){
                                            eval(item);
                                        });
                                    });
                                }
                            }
                            $('.crm-rebpm-extract-phone-button').hide();
                        }, 3000);

                    }
                });
                return !1;
            });
        }
    },

    // Вызывается через factory
    getRealtyObjectData : function (params) {
        let data = {
            allow : false,
            pageData : {}
        };

        let $rltbaseObjectsData = $('.rltbase-object-data');

        data.pageData.advertisement_id = $rltbaseObjectsData.data('advertisement-id') || null;
        data.pageData.advertisement_url = $rltbaseObjectsData.data('advertisement-url') || null;
        data.pageData.native_id = $rltbaseObjectsData.data('native-id') || null;
        data.pageData.source_url = $rltbaseObjectsData.data('source-url') || null;
        data.pageData.source = $rltbaseObjectsData.data('source') || null;
        data.pageData.region = $rltbaseObjectsData.data('region') || null;

        if(
            data.pageData.advertisement_id &&
            data.pageData.advertisement_url &&
            data.pageData.native_id &&
            data.pageData.source_url &&
            data.pageData.source &&
            data.pageData.region
        ) {
            data.allow = true;
        }


        data.tab_id = params.tab_id;
        let port = chrome.runtime.connect();
        port.postMessage({method:'setRealtyObjectData', data : data});
    },

    onExtensionInit : function () {
        if(window.location.href.indexOf('admin/rltbase_connector/rltbase_parsed') !== -1) {
            $('.rltbase-subscribe-blocks').removeClass('hidden');
            $('.install-rebpm-button').remove();
        }

        let script = document.createElement('script');
        script.src = chrome.runtime.getURL('inject.js')
        document.getElementsByTagName("body")[0].appendChild(script);

        $('body').on('rebpm-extension-extract-phone',function (event) {
            let el = null;
            if(event.detail && event.detail.el_id) {
                el = $('body').find(`#${event.detail.el_id}`);
            }
            if(el && el.length > 0) {
                $(el).find('.extract-phone_button').hide();
                $(el).find('.extract-phone_loader').show();
            }
            chrome.runtime.sendMessage({from : 'content', subject : 'extractRealtyObjectPhone', data : event.detail}).then((response) => {
                if(el && el.length > 0) {
                    $(el).find('.extract-phone_button').show();
                    $(el).find('.extract-phone_loader').hide();
                }
                $('body')[0].dispatchEvent(new CustomEvent("rebpm-extension-extracted-phone", {
                    detail : {
                        data : event.detail,
                        result : response
                    }
                }));
            });
        });

        this.realtimeSubsUpdate();
        setInterval(this.realtimeSubsUpdate,20000);
    },

    realtimeSubsUpdate : function () {
        chrome.runtime.sendMessage({from : 'content', subject : 'getSubscriptions'}).then((response) => {
            let subs = null;
            if(response.hasOwnProperty('subscriptions')){
                subs = response.subscriptions;
            }
            if(subs === null){
                return false;
            }
            let totalCount = 0;
            for (let [key, sub] of Object.entries(subs)) {
                let option = $('#parsed-subscription-id').find(`[data-id="${sub.id}"]`);
                let text = `${sub.title} [${sub.count}]`;
                option.text(text);
                totalCount += sub.count;
            }

            let $counter = $('#rltbase-connector-header-button').find('.counter-new-objects');
            if(totalCount > 0) {
                $counter.text(totalCount);
                $counter.closest('a').addClass('alert-new-objects');
            } else {
                $counter.text('');
                $counter.closest('a').removeClass('alert-new-objects');
            }

        });
    }

};

RESPY_PageParser.init();
