$(function () {
	// let defaults = ;

	var svgElement = function(tag, attrs) {
		return $(
			document.createElementNS('http://www.w3.org/2000/svg', tag)
		).attr(attrs)
	};

	$.fn.peity.register('priceline', {
			delimiter: ",",
			fill: false,
			height: 16,
			min: 0,
			stroke: "#4d89f9",
			textSelector : '.chart-text',
			realtyObjectSelector : '',
			textOffsetX : 5,
			textOffsetY : 5,
			strokeWidth: 1,
			pointRadius : 4,
			padding: 10,
			width: 32
		}, function (opts) {
			let that = this;
			let points = [];
			var date = $(this.$el).data('date');
			var values = $(this.$el).data('price');
			if (values.length == 1) values.push(values[0]);
			var max = Math.max.apply(Math, opts.max == undefined ? values : values.concat(opts.max))
				, min = Math.min.apply(Math, opts.min == undefined ? values : values.concat(opts.min))

			var $svg = this.prepare(opts.width + opts.padding, opts.height + opts.padding)
				, strokeWidth = opts.strokeWidth
				, width = $svg.width() - opts.padding
				, height = $svg.height() - opts.padding
				, diff = max - min

			var xScale = this.x = function (input) {
				return input * (width / (values.length - 1)) + opts.padding / 2
			}

			var yScale = this.y = function (input) {
				var y = height

				if (diff) {
					y -= ((input - min) / diff) * height
				}

				return y + strokeWidth / 2 + opts.padding / 2
			};

			var priceFormat = function(data){
				var price = Number.prototype.toFixed.call(parseFloat(data) || 0, 0);
				return price.replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1 ");
			};

			var zero = yScale(Math.max(min, 0))
				, coords = [0, zero]

			for (var i = 0; i < values.length; i++) {

				let cx = xScale(i);
				let cy = yScale(values[i]);

				coords.push(cx, cy);

				points.push(svgElement('circle', {
					cx:cx,
					cy:cy,
					r: opts.pointRadius,
					'data-date' : date[i],
					'data-price' : values[i],
					'svg-name' : 'price-point',
					fill : '#08c'
				}));
			}

			coords.push(width, zero);

			if (opts.fill) {
				$svg.append(
					svgElement('polygon', {
						fill: opts.fill,
						points: coords.join(' ')
					})
				)
			}

			if (strokeWidth) {
				$svg.append(
					svgElement('polyline', {
						fill: 'none',
						points: coords.slice(2, coords.length - 2).join(' '),
						stroke: opts.stroke,
						'stroke-width': strokeWidth,
						'stroke-linecap': 'square'
					})
				);
			}

			$.each(points,function (index,point) {
				$svg.append(point);
			});

			$(opts.realtyObjectSelector).find('[svg-name="price-point"]').mouseenter(function (e) {
				let $element = $(e.target);

				let x = Number($element.attr('cx')) + opts.textOffsetX;
				let y = Number($element.attr('cy'))+ opts.textOffsetY;

				let date = $element.data('date');
				let price = $element.data('price');

				let $text = $(that.$el).find(opts.textSelector);

				$text.css({
					left: x + 'px',
					top : y + 'px',
					display : 'block'
				});
				$text.find('.chart-text-date').text(date);
				$text.find('.chart-text-price').text(priceFormat(price));
			});

			$(opts.realtyObjectSelector).find('[svg-name="price-point"]').mouseleave(function (e) {
				let $text = $(that.$el).find(opts.textSelector);
				$text.css({
					left: 0 + 'px',
					top : 0 + 'px',
					display : 'none'
				});
			});

		}
	);
});
