'use strict';

let RESPY_Popup = {

    rltbaseDomain : 'rltbase.ru',

    tokenAllow : '7345fghfgh3pqdfGGddf',

    init : function () {
        RESPY_Popup_UI.tabs();
        RESPY_Popup_Parsing.init();
        RESPY_Popup_Settings.init();
        RESPY_Popup_Subscriptions.init();
        RESPY_Popup_Notifications.init();

        RESPY_Popup_UI.hideParsingButton(true);

        chrome.tabs.query({
            active: true,
            currentWindow: true
        }, (tabs) => {
            chrome.runtime.sendMessage({from : 'popup', subject : 'getAllowParseData', tabId: tabs[0].id}, (response) => {
                RESPY_Popup_Parsing.allowParse(response);
            });
            chrome.runtime.sendMessage({from : 'popup', subject : 'getRealtyObjectData', tabId: tabs[0].id}, (response) => {
                RESPY_Popup_Parsing.allowParse(response);
            });
        });
    }
};
function reloadPageParsed(url,hash){
    window.location.href = url;
    window.location.hash = hash;
    window.location.reload();
}

let RESPY_Popup_Subscriptions = {

    subscriptions : {},

    init : function() {
        RESPY_Popup_UI.setSubscriptionsInfo('');
        // Затем делаем запрос за обновлением подписки.
        // Таким образом добьёмся быстрого отображения подписок
        chrome.runtime.sendMessage({from : 'popup', subject : 'getSubscriptions'}, (response) => {
            if(response.subscriptions && response.subscriptions !== null) {
                this.subscriptions = response.subscriptions;
                this.renderSubscriptions(this.subscriptions);
            } else if(response.errors) {
                RESPY_Popup_UI.setSubscriptionsInfo(response.errors.join(', '));
            } else {
                RESPY_Popup_UI.setSubscriptionsInfo('Подписок не найдено');
            }
        });

        this.listeners();
    },

    listeners : function() {
        $('#tab-subscriptions .subscription-action-objects').click( function (event) {
            event.preventDefault();
            let id = $(this).closest('.subscription').attr('data-subscription-id');
            let crm = RESPY_Popup_Settings.get('respy_settings_crm');
            let hash = `#rltbase-subscribtion=${id}`;
            let url = `${crm}/admin/rltbase_connector/rltbase_parsed/${hash}`;
            chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
                let currentTab = tabs[0];
                if(currentTab.url.indexOf(`${crm}/admin/rltbase_connector/rltbase_parsed`) !== -1) {
                    // Если сейчас активна вкладка с парсером, мы обновляем её
                    chrome.tabs.executeScript(currentTab.id,{code : `
                        window.location.href = "${url}";
                        window.location.hash = "${hash}";
                        window.location.reload();`
                    });
                    chrome.scripting.executeScript({
                        target: {tabId: currentTab.id},
                        func : reloadPageParsed //TODO ADD ARGS
                    });
                } else {
                    // Иначе создаём новую вкладку с парсером
                    chrome.tabs.create({active: true, url: url});
                }
            });
        });

        $('#tab-subscriptions .subscription-action-view').click( function (event) {
            event.preventDefault();
            let target = $(this).closest('.subscription').find('.subscription-config');
            $(target).stop().slideToggle('fast');
        });

        $('#tab-subscriptions .subscription-action-edit').click( function (event) {
            event.preventDefault();
        });

        $('#tab-subscriptions .subscription-action-delete').click( function (event) {
            event.preventDefault();
            $.confirm({
                theme: 'light',
                title: "Подтвердите действие",
                content: 'Вы уверены, что хотите отписаться от подписки?',
                backgroundDismiss: true,
                boxWidth: '80%',
                useBootstrap: false,
                buttons: {
                    confirm: {
                        text : 'Да',
                        btnClass: 'btn-red',
                        action : function () {
                            RESPY_Popup_Subscriptions.deleteSubscription(event.target);
                        }
                    },
                    cancel: {
                        text : 'Отмена',
                        action : function () {
                            this.close();
                        }
                    }
                },
                onOpenBefore: function () {
                    $('#app').addClass('confirm-form');
                },
                onClose: function () {
                    $('#app').removeClass('confirm-form');
                },
            });
        });
    },

    renderSubscriptions : function(subscriptions) {
        $("#tab-subscriptions .subscription").remove();
        for(let key in subscriptions) {
            let subscription = subscriptions[key];
            let $subscriptionBlock = $("#tab-subscriptions .subscription-template").clone(true);
            $subscriptionBlock.removeClass('subscription-template').addClass('subscription');

            $subscriptionBlock.attr('data-subscription-id',subscription.id);
            $subscriptionBlock.find('.subscription-title')
                .text(subscription.title)
                .attr('title',subscription.title);
            $subscriptionBlock.find('.subscription-count').text(subscription.count);
            this.renderSubscriptionConfig($subscriptionBlock.find('.subscription-config'),subscription.config);

            $("#tab-subscriptions .subscription-wrapper").append($subscriptionBlock);
        }
    },

    renderSubscriptionConfig: function (container,config) {
        let order = {
            'region': null,
            'district': null,
            'locality': null,
            'sub_locality': null,
            'street': null,
            'house': null,
            'category' : null,
            'deal_type': null,
            'rooms': null,
            'price_from': null,
            'price_to': null,
            'price_area_from': null,
            'price_area_to': null,
            'area_from': null,
            'area_to': null,
            'floor_from': null,
            'floor_to': null,
            'floors_total_from': null,
            'floors_total_to':null
        }
        for(let [key, value] of Object.entries(config)) {
            let title = RESPY_Translator.translate(key);
            if(key === 'is_agency') {
                value = value ? 'Да' : 'Нет';
            }
            let $item = $('<div>', { 'class' : 'subscription-config-item'});
            $('<div>', { 'class' : 'subscription-config-key', text: title}).appendTo($item);
            $('<div>', { 'class' : 'subscription-config-value', text: value}).appendTo($item);
            order[key] = $item;
        }

        for(let [key, $item] of Object.entries(order)) {
            $($item).appendTo(container);
        }
    },

    deleteSubscription : async function (target) {

        let id = $(target).closest('.subscription').attr('data-subscription-id');
        let respy_subscription_token = RESPY_Popup_Settings.get('respy_subscription_token');
        let respy_subscription_domain = RESPY_Popup_Settings.get('respy_subscription_domain');

        if(!id || !respy_subscription_token || !respy_subscription_domain) {
            RESPY_Popup_UI.setSubscriptionsInfo('Что-то пошло не так. Обратитесь в службу техподдержки: help@rebpm.ru', 5000);
            return false;
        }

        try {
            RESPY_Popup_UI.setSubscriptionsInfo('Загрузка... Пожалуйста, подождите');
            let url = `http://${RESPY_Popup.rltbaseDomain}:3031/subscriptions/unsrieb/${id}/?token=${respy_subscription_token}&domain=${respy_subscription_domain}`;
            let response = await fetch(url);
            response = await response.json();
            if(response.errors && Array.isArray(response.errors)) {
                RESPY_Popup_UI.setSubscriptionsInfo('Ошибка отписки: ' + response.errors.join(', '));
            }

            if(response.deleted) {
                RESPY_Popup_UI.setSubscriptionsInfo(`Вы успешно отписались от подписки №${response.deleted}`,5000);
                $(target).closest('.subscription').remove();
            }
        } catch (e) {
            RESPY_Popup_UI.setSubscriptionsInfo('Что-то пошло не так при отписке. Обратитесь в службу техподдержки: help@rebpm.ru', 5000);
        }
    }
};

let RESPY_Popup_Parsing = {

    objectData : {},

    init : function () {
        this.listeners();
    },

    listeners : function () {
        this.listenParseButton();
    },

    // callback для функции определения, находимся ли мы на странице объекта. Получаем данные об объекте
    allowParse : function (data) {
        if(data && typeof data == "object" && data.hasOwnProperty('allow') && data.allow) {
            RESPY_Popup_Parsing.objectData = data.pageData;
            RESPY_Popup_UI.showParsingButton();
            $('a[href="#tab-parsing"]').closest('li').show();
            if( data.hasOwnProperty('allowExtractPhone') && data.allowExtractPhone) {
                $('a[href="#tab-extract-phone"]').closest('li').show();
            }
        }

    },

    listenParseButton : function () {
        // Нажание на кнопку "Добавить в базу"
        $('#parseObjectButton').click( (event) => {
            event.preventDefault();

            RESPY_Popup_UI.setParsingError('');

            let crm = RESPY_Popup_Settings.get('respy_settings_crm');

            // Отслеживаем возможные ошибки
            if(!this.objectData) {
                RESPY_Popup_UI.setParsingError('Увы, мы не смогли получить данные со страницы');
                return false;
            }

            if(!crm) {
                RESPY_Popup_UI.setParsingError('Введите адрес вашей CRM во вкладке "Настройки"');
                return false;
            }

            RESPY_Popup_UI.setParsingInfo('Загрузка объекта... (Вы можете закрыть это окно)');
            // Если всё нормально - выполняем запрос на rltbase
            this.objectData.crm = crm;
            RESPY_Popup_UI.hideParsingButton();
            chrome.runtime.sendMessage({
                from : 'popup',
                subject : 'loadRltbase',
                data : this.objectData,
            }).then( (response) => {
                RESPY_Popup_UI.setParsingInfo('');
                if(response.hasOwnProperty('result') && response.result) {
                    // Объект упешно добавлен
                    RESPY_Popup_UI.setParsingInfo(`Объект успешно добавлен в базу (№${response.objectId})`);
                } else {
                    if(response.hasOwnProperty('wait') && response.wait) {
                        // Создано задание на парсинг ждем перепарсинга
                        RESPY_Popup_UI.setParsingInfo(`${response.wait}`);
                        RESPY_Popup_UI.showParsingButton();
                    }else {

                        // Запрос с ошибкой. Выводим на экран.
                        RESPY_Popup_UI.showParsingButton();
                        if(response.hasOwnProperty('errors')) {
                            RESPY_Popup_UI.setParsingError(response.errors.join(';'));
                        } else if(response.hasOwnProperty('error')) {
                            RESPY_Popup_UI.setParsingError(response.error);
                        } else {
                            RESPY_Popup_UI.setParsingError('Неизвестная ошибка');
                        }
                    }
                }
            });
        });
    }
};

let RESPY_Popup_Notifications = {

    objectData : {},

    notifications : [],

    init : function () {
        this.loadNotifications();
        this.listeners();
    },

    listeners : function () {
        $('#subscribeObjectChange').click(async function (event) {
            // Пользователь подписывается на объект.
            let respy_subscription_user_id = RESPY_Popup_Settings.get('respy_subscription_user_id');
            let respy_subscription_token = RESPY_Popup_Settings.get('respy_subscription_token');
            let respy_subscription_domain = RESPY_Popup_Settings.get('respy_subscription_domain');

            if(!respy_subscription_user_id || !respy_subscription_token || !respy_subscription_domain) {
                RESPY_Popup_UI.setSubscribeObjectChangeInfo('Подписки не настроены. Перейдите на вкладку настройки, введите адрес своей CRM и нажмите Сохранить. Затем повторите попытку.');
                return false;
            }

            RESPY_Popup_Notifications.objectData['user_id'] = respy_subscription_user_id;

            try {
                RESPY_Popup_UI.setSubscribeObjectChangeInfo('Загрузка... Пожалуйста, подождите');
                let url = `http://${RESPY_Popup.rltbaseDomain}:3031/object_subscriptions/?token=${respy_subscription_token}&domain=${respy_subscription_domain}`;
                let response = await $.ajax(url, {
                    'async' : true,
                    'type' : "PUT",
                    'data' : JSON.stringify(RESPY_Popup_Notifications.objectData),
                    'contentType' : 'application/json',
                    'dataType' : 'json'
                });

                if(response.errors.length > 0) {
                    let error = response.errors;
                    if(Array.isArray(response.errors)) {
                        error = response.errors.join(', ');
                    }
                    RESPY_Popup_UI.setSubscribeObjectChangeInfo(`Ошибка подписки: ${error}`);
                }

                if(response.data && response.data.ObjectChangeSubscription) {
                    RESPY_Popup_UI.setSubscribeObjectChangeInfo('Вы успешно подписались на объект');
                    $(event.target).attr('disabled','disabled');
                }
            } catch (e) {
                RESPY_Popup_UI.setSubscribeObjectChangeInfo('Что-то пошло не так при подписке. Обратитесь в службу техподдержки: help@rebpm.ru');
            }

        });

        $('#tab-notifications .notification-action-read').click( function (event) {
            event.preventDefault();
            RESPY_Popup_Notifications.readed(event.target);
        });

        $('#tab-notifications .notification-action-info').click( function (event) {
            event.preventDefault();
            let target = $(this).closest('.notification').find('.notification-info');
            $(target).stop().slideToggle('fast');
        });

        $('#tab-notifications .notification-action-delete').click( function (event) {
            event.preventDefault();
            $.confirm({
                theme: 'light',
                title: "Подтвердите действие",
                content: 'Вы уверены, что хотите отписаться от подписки?',
                backgroundDismiss: true,
                boxWidth: '80%',
                useBootstrap: false,
                buttons: {
                    confirm: {
                        text : 'Да',
                        btnClass: 'btn-red',
                        action : function () {
                            RESPY_Popup_Notifications.unscribe(event.target);
                        }
                    },
                    cancel: {
                        text : 'Отмена',
                        action : function () {
                            this.close();
                        }
                    }
                },
                onOpenBefore: function () {
                    $('#app').addClass('confirm-form');
                },
                onClose: function () {
                    $('#app').removeClass('confirm-form');
                },
            });
        });

        $('#tab-notifications .notification-group-label').click( function (event) {
            event.preventDefault();
            let target = $(this).closest('.notification-group').find('.notification-group-content');
            $(target).slideToggle('fast');
        });
    },

    // callback для функции определения, находимся ли мы на странице объекта в системе, который был добавлен через парсер. Получаем данные
    setRealtyObjectData : function (data) {
        if(data && typeof data == "object" && data.hasOwnProperty('allow') && data.allow) {
            RESPY_Popup_Notifications.objectData = data.pageData;
            RESPY_Popup_UI.showSubscribeObjectChangeBlock();
        }
    },

    loadNotifications : async function () {
        // При открытие popup сначала загружаем уже имеющиеся подписки
        chrome.runtime.sendMessage({from : 'popup', subject : 'getObjectChangeSubscriptions'}, (response) => {
            if(response.notifications && response.subscriptions !== null) {
                this.renderNotifications(response.notifications);
            } else if(response.errors) {
                RESPY_Popup_UI.setNotificationInfo(response.errors.join(', '));
            } else {
                RESPY_Popup_UI.setNotificationInfo('Подписки не настроены');
            }
        });
    },

    renderSubscribeNotification : function() {
        let $container = $('.subscribe-object-change-container');
        $container.find('.notification-action-advertisement').attr('href', this.objectData.advertisement_url);
        $container.find('.notification-action-source').attr('href', this.objectData.source_url);
        $container.find('.subscribe-object-change-text')
            .html(`Объект № <span class="subscribe-object-change-id">${this.objectData.advertisement_id}</span>`);
        $container.find('.subscribe-object-change-id').attr('title',`
            Внешний ID: ${this.objectData.native_id}
Регион: ${this.objectData.region}`);
    },

    renderNotifications :function (notifications) {
        $("#tab-notifications .subscription").remove();
        if(notifications.errors.length > 0) {
            RESPY_Popup_UI.setNotificationInfo(notifications.errors.join(', '));
            return ;
        }
        for(let key in notifications['data']) {
            let notification = notifications['data'][key]['ObjectChangeSubscription'];
            let $notificationBlock = $("#tab-notifications .notification-template").clone(true);
            $notificationBlock.removeClass('notification-template').addClass('notification');
            $notificationBlock.attr('data-region',notification.object.region);
            $notificationBlock.attr('data-id',notification.object.id);
            $notificationBlock.find('.notification-title').html(RESPY_Popup_Notifications.renderNotificationTitle(notification));
            $notificationBlock.find('.notification-icon').html(RESPY_Popup_Notifications.renderNotificationIcon(notification));

            $notificationBlock.find('.notification-info .notification-action-source').attr('href', notification.object.url);
            $notificationBlock.find('.notification-info .notification-action-advertisement')
                .attr('href', notification.user.advertisement.url)
                .attr('title',`Перейти к объекту #${notification.user.advertisement.id} в Вашей CRM`)
            $notificationBlock.find('.notification-icon').html(RESPY_Popup_Notifications.renderNotificationIcon(notification));

            if(notification.user.state === "new") {
                let $group = $("#tab-notifications .notification-wrapper > .new-notification-group > .notification-group-content");
                $group.find('.notification-group-content-placeholder').remove();
                $group.append($notificationBlock);
                RESPY_Popup_UI.setHasNewNotifications(true);
            } else {
                let $group = $("#tab-notifications .notification-wrapper > .readed-notification-group > .notification-group-content");
                $group.find('.notification-group-content-placeholder').remove();
                $group.append($notificationBlock);
            }

            this.renderNotificationInfo($notificationBlock.find('.notification-info > .notification-info-content'),notification);
        }
    },

    renderNotificationTitle : function (notification) {
        if(!notification.status || !notification.user.state) {
            return 'Без изменений';
        }
        if(notification.status === 'price_changed') {
            return 'Изменилась цена';
        }
        if(notification.status === 'deleted') {
            return 'Объект снят с площадки';
        }
    },

    renderNotificationIcon : function (notification) {
        if(!notification.status || !notification.user.state) {
            return '<i class="icon" title="Без изменений"><img src="img/ban.png"></i>';
        }

        if(notification.status === 'price_changed') {
            return '<i class="icon" title="Изменилась цена"><img src="img/chart.png"></i>';
        }

        if(notification.status === 'deleted') {
            return '<i class="icon" title="Объект снят с площадки"><img src="img/delete.png"></i>';
        }
    },

    renderNotificationInfo : function (target, notification) {
        if(notification.status === 'price_changed' && notification.object && notification.object.price_log) {
            this.initChart(target, notification.object.price_log, notification.object);
        }
        RESPY_Popup_Notifications.renderNotificationObjectInfo($(target).find('.subscription-config'),notification.object);
    },

    renderNotificationObjectInfo: function (container,object) {
        let order = {
            'source' : null,
            'updated' : null,
            'category' : null,
            'deal_type': null,
            'region': null,
            'locality': null,
            'street': null,
            'house': null,
            'rooms': null,
            'price': null,
            'area': null,
            'floor': null,
            'floors_total': null
        }
        for(let [key, value] of Object.entries(object)) {
            let title = RESPY_Translator.translate(key);
            if(key === 'is_agency') {
                value = value ? 'Да' : 'Нет';
            }
            let $item = $('<div>', { 'class' : 'subscription-config-item'});
            $('<div>', { 'class' : 'subscription-config-key', text: title}).appendTo($item);
            $('<div>', { 'class' : 'subscription-config-value', text: value}).appendTo($item);
            if(order.hasOwnProperty(key)) {
                order[key] = $item;
            }
        }

        for(let [key, $item] of Object.entries(order)) {
            $($item).appendTo(container);
        }
    },

    initChart : function (target, price_log, object) {
        var data = JSON.parse(price_log);
        if(data.price_log !== undefined) {
            // Фикс для нового формата поля
            data = data.price_log;
        }
        var date = [];
        var price = [];

        data.forEach(function (item) {
            date.push(item.date);
            price.push(item.price);
        });
        var element = `.notification[data-id='${object.id}'][data-region='${object.region}'] .price-chart`;

        $(element).data('date',date);
        $(element).data('price',price);

        if(typeof $().peity !== 'undefined') {
            $(element).peity("priceline",{
                height : 50,
                width: 170,
                pointRadius : 3,
                padding: 40,
                margin: 10,
                textSelector : '.chart-text',
                realtyObjectSelector : `.notification[data-id='${object.id}'][data-region='${object.region}']`
            });
        }

        $(element).show();
    },

    readed : async function (target) {
        let notification = $(target).closest('.notification');
        let id = notification.attr('data-id');
        let region = notification.attr('data-region');
        let respy_subscription_user_id = RESPY_Popup_Settings.get('respy_subscription_user_id');
        let respy_subscription_domain = RESPY_Popup_Settings.get('respy_subscription_domain');

        if(!id || !respy_subscription_token || !respy_subscription_domain) {
            RESPY_Popup_UI.setNotificationInfo('Что-то пошло не так. Обратитесь в службу техподдержки: help@rebpm.ru', 5000);
            return false;
        }

        try {
            RESPY_Popup_UI.loadingStart('notification_readed',$(target));
            let url = `http://${RESPY_Popup.rltbaseDomain}:3031/object_subscriptions/state/?token=${RESPY_Popup.tokenAllow}&domain=${respy_subscription_domain}&user_id=${respy_subscription_user_id}`;
            /*let response = await $.ajax(url, {
                'async' : true,
                'type' : "POST",
                'data' : JSON.stringify({
                    id : id,
                    region : region,
                    state : 'readed'
                }),
                'contentType' : 'application/json',
                'dataType' : 'json'
            });*/
            let response = await fetch(url, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    id : id,
                    region : region,
                    state : 'readed'
                })
            });
            response = await response.json();
            RESPY_Popup_UI.loadingEnd('notification_readed', $(target));
            if(response.errors && Array.isArray(response.errors) && response.errors.length > 0) {
                RESPY_Popup_UI.setNotificationInfo('Ошибка: ' + response.errors.join(', '));
            }
            if(response.data) {
                notification.find('.notification-action-read').remove();
            }
        } catch (e) {
            RESPY_Popup_UI.setNotificationInfo('Что-то пошло не так. Обратитесь в службу техподдержки: help@rebpm.ru', 5000)
            RESPY_Popup_UI.loadingEnd('notification_readed', $(target));
        }
    },

    unscribe : async function (target) {
        let notification = $(target).closest('.notification');
        let id = notification.attr('data-id');
        let region = notification.attr('data-region');
        let respy_subscription_user_id = RESPY_Popup_Settings.get('respy_subscription_user_id');
        let respy_subscription_domain = RESPY_Popup_Settings.get('respy_subscription_domain');

        if(!id || !respy_subscription_token || !respy_subscription_domain) {
            RESPY_Popup_UI.setNotificationInfo('Что-то пошло не так. Обратитесь в службу техподдержки: help@rebpm.ru', 5000);
            return false;
        }

        try {
            RESPY_Popup_UI.loadingStart('notification_unscribe',$(target));
            let url = `http://${RESPY_Popup.rltbaseDomain}:3031/object_subscriptions/?token=${RESPY_Popup.tokenAllow}&domain=${respy_subscription_domain}&user_id=${respy_subscription_user_id}`;
            let response = await $.ajax(url, {
                'async' : true,
                'type' : "DELETE",
                'data' : JSON.stringify({
                    id : id,
                    region : region
                }),
                'contentType' : 'application/json',
                'dataType' : 'json'
            });
            RESPY_Popup_UI.loadingEnd('notification_unscribe', $(target));
            if(response.errors && Array.isArray(response.errors) && response.errors.length > 0) {
                RESPY_Popup_UI.setNotificationInfo('Ошибка отписки: ' + response.errors.join(', '));
            }
            if(response.data) {
                notification.remove();
            }
        } catch (e) {
            RESPY_Popup_UI.setNotificationInfo('Что-то пошло не так при отписке. Обратитесь в службу техподдержки: help@rebpm.ru', 5000)
            RESPY_Popup_UI.loadingEnd('notification_unscribe', $(target));
        }
    }
};

let RESPY_Popup_UI = {
    $parseObjectButton : $('#tab-parsing #parseObjectButton'),
    $parsingInfo : $('#tab-parsing .parsing-info'),
    $parsingError : $('#tab-parsing .parsing-error'),
    $parsingButtonPlaceholder : $('#tab-parsing .parsing-button-placeholder'),
    $settingsInfo : $('#tab-settings .settings-info'),
    $subscriptionsInfo : $('#tab-subscriptions .subscriptions-info'),
    $subscribeObjectChangeBlock : $('#tab-subscriptions .subscribe-object-change-container'),
    $subscribeObjectChangeInfo : $('#tab-subscriptions .subscribe-object-change-topper-text'),
    $notificationsInfo: $('#tab-notifications .notifications-info'),

    loading: {},
    tabs : function () {
        // https://snipp.ru/jquery/tabs
        let tabs = $('#app .tabs-items > div');
        tabs.hide().filter(':first').show();

        // Клики по вкладкам.
        $('#app .tabs-nav a').click(function(){
            tabs.hide();
            tabs.filter(this.hash).show();
            $('#app .tabs-nav a').removeClass('active');
            $(this).addClass('active');
            return false;
        }).filter(':first').click();

        // Клики по якорным ссылкам.
        $('.tabs-target').click(function(){
            $('#app .tabs-nav a[href=' + $(this).data('id')+ ']').click();
        });
    },
    loadingStart : function (key,element) {
        this.loading[key] = $(element).attr('src');
        $(element).attr('src','img/loading.gif');
    },
    loadingEnd : function (key,element) {
        $(element).attr('src',this.loading[key]);
    },
    showSubscribeObjectChangeBlock : function () {
        this.$subscribeObjectChangeBlock.show();
        RESPY_Popup_Notifications.renderSubscribeNotification();
    },
    setSubscribeObjectChangeInfo : function (value) {
        this.$subscribeObjectChangeInfo.text(value);
    },
    setNotificationInfo : function (value) {
        this.$notificationsInfo.text(value);
    },
    setHasNewNotifications : function (state) {
        if(state) {
            $('a[href="#tab-notifications"] > .new-notifications').show();
        } else {
            $('a[href="#tab-notifications"] > .new-notifications').hide();
        }
    },
    showParsingButton : function () {
        this.$parseObjectButton.show();
        this.$parsingButtonPlaceholder.text('');
    },
    hideParsingButton : function (placeholder = false) {
        this.$parseObjectButton.hide();
        if(placeholder) {
            this.$parsingButtonPlaceholder.text('На текущей странице мы не смогли найти объект недвижимости. (Возможно страница еще не загрузилась. Когда страница польностью загрузится - попробуйте снова)');
        }
    },
    setParsingInfo : function(value) {
        this.$parsingInfo.text(value);
    },
    setParsingError : function(value) {
        this.$parsingError.text(value);
    },
    setSubscriptionsInfo : function(value, timeout = null) {
        this.$subscriptionsInfo.text(value);
        if(timeout) {
            setTimeout(() => this.$subscriptionsInfo.text(''), timeout);
        }
    },
    setInputValue : function (tab, input, value) {
        $(`#tab-${tab}`).find(input).val(value);
    },
    getInputValue : function (tab, input) {
        return $(`#tab-${tab}`).find(input).val();
    },
    setSettingsInfo : function (value) {
        this.$settingsInfo.text(value);
        setTimeout(() => this.$settingsInfo.text(''), 5000);
    }
};

let RESPY_Popup_Settings = {

    settings : {},

    allowSettings : [
        'respy_settings_crm',
        'respy_subscription_domain',
        'respy_subscription_token',
        'respy_subscription_user_id',
        'respy_custom_server'
    ],

    init : async function () {
        await this.set();

        $('#tab-settings #saveSettings').click( (event) => {
            event.preventDefault();
            this.save();
            if(!this.settings['respy_subscription_domain'] || !this.settings['respy_subscription_token']) {
                // Подписки еще не настроены. Пробуем настроить с текущим доменом
                this.autoSetSubscriptions();
            } else {
                RESPY_Popup_UI.setSettingsInfo('Сохранено');
            }
        });

        $('#tab-settings #autoSettings').click( (event) => {
            event.preventDefault();
            this.autoSetSubscriptions();
        });

        $('#tab-settings [jq-toggle]').click( function (event) {
            event.preventDefault();
            let target = $(this).attr("jq-toggle");
            $(target).slideToggle('300');
        });

    },

    save : async function() {
        this.allowSettings.forEach( (key,index,array) => {
            let value = RESPY_Popup_UI.getInputValue('settings',`#${key}`);
            if(key === 'respy_settings_crm') {
                value = this._prepareSettingCrm(value);
            }
            this.settings[key] = value;
        });
        await RESPY_Popup_Settings.chromeStorageSet(this.settings);
        await this.set();
    },

    set : async function() {
        // Получаем все настройкм
        let result = await RESPY_Popup_Settings.chromeStorageGet(this.allowSettings);
        this.allowSettings.forEach( (key) => {
            this._setSetting(key,result[key]);
        });
    },

    autoSetSubscriptions : async function() {
        let crm = RESPY_Popup_Settings.get('respy_settings_crm');

        if(!crm) {
            RESPY_Popup_UI.setSettingsInfo('Введите адрес вашей CRM (Возможно вы не нажали кнопку "Сохранить")');
            return false;
        }

        RESPY_Popup_UI.setSettingsInfo('Производится настройка подписок...');

        let url = `${crm}/admin/rltbase_subscription/rltbase_subscription/auto_settings.json`;
        try {
            var response = await fetch(url);
            response = await response.json();
            if(response.hasOwnProperty('errors') && response.errors) {
                RESPY_Popup_UI.setSettingsInfo(response.errors.join(', '));
            }

            if(response.hasOwnProperty('result') && response.result) {
                if(response.result.domain) {
                    this._setSetting('respy_subscription_domain',response.result.domain);
                }
                if(response.result.token) {
                    this._setSetting('respy_subscription_token',response.result.token);
                }
                if(response.result.user_id) {
                    this._setSetting('respy_subscription_user_id',response.result.user_id);
                }
            }

            RESPY_Popup_UI.setSettingsInfo('Подписки успешно настроены');

            this.save();
            RESPY_Popup_Subscriptions.init();

        } catch (e) {
            RESPY_Popup_UI.setSettingsInfo('Ошибка запроса к вашей CRM (Возможно не включен плагин подписок, либо вы не авторизованы в системе)');
        }

    },

    get : function(key) {
        if(this.settings.hasOwnProperty(key)) {
            return this.settings[key];
        }
        return null;
    },

    _prepareSettingCrm : function (value) {
        let link = document.createElement('a');
        link.href = value;
        return link.protocol+'//'+link.hostname;
    },

    _setSetting : function (key,value) {
        this.settings[key] = value;
        RESPY_Popup_UI.setInputValue('settings',`#${key}`,value);
    },

    chromeStorageGet : async function (settings) {
        return await new Promise(function (resolve, reject) {
            chrome.storage.sync.get(settings, function (response) {
                resolve(response);
            });
        });
    },

    chromeStorageSet : async function (settings) {
        return await new Promise(function (resolve, reject) {
            chrome.storage.sync.set(settings, function (response) {
                resolve(response);
            });
        });
    }
};

document.addEventListener('DOMContentLoaded' ,function () {
    RESPY_Popup.init();
});
