'use strict';

const rltbaseDomain = 'rltbase.ru';
const tokenAllow = '7345fghfgh3pqdfGGddf';
let customRltbaseDomain = false;


chrome.action.setBadgeBackgroundColor({color:'#FF0000'}); //Устанавливает фон текста поверх иконки

chrome.runtime.onMessage.addListener( (msg, sender, response) => {
    if ((msg.from === 'popup') && (msg.subject === 'loadRltbase') && msg.data) {
        loadRltbase(msg.data).then((result) => {
            response(result);
        });
        return true;
    }
    if ((msg.from === 'content') && (msg.subject === 'extractPhone') && msg.data) {
        extractPhone(msg.data).then(response);
        return true;
    }
    if ((msg.from === 'content') && (msg.subject === 'extractPhoneSave') && msg.data) {
        extractPhoneSave(msg.data).then(response);
        return true;
    }
    if ((msg.from === 'popup') && (msg.subject === 'getSubscriptions')) {
        RESPY_Background_Subscriptions.getSubscriptions().then(function (resolve) {
            response({
                subscriptions : RESPY_Background_Subscriptions.subscriptions,
                errors : RESPY_Background_Subscriptions.errors
            });
        });
        return true;
    }
    if ((msg.from === 'popup') && (msg.subject === 'getObjectChangeSubscriptions')) {
        RESPY_Background_Notifications.getObjectChangeSubscriptions().then(function (resolve) {
            response({
                notifications: resolve,
                errors: RESPY_Background_Notifications.errors
            });
        });
        return true;
    }

    if ((msg.from === 'popup') && (msg.subject === 'getAllowParseData')) {
        response(RESPY_Background.getAllowParseData(msg.tabId));
        return true;
    }
    if ((msg.from === 'popup') && (msg.subject === 'getRealtyObjectData')) {
        response(RESPY_Background.getRealtyObjectData(msg.tabId));
        return true;
    }
    if ((msg.from === 'content') && (msg.subject === 'getSubscriptions')) {
        // Тут запрос не делаем. Просто получаем из сохранённых данных
        response({
            subscriptions : RESPY_Background_Subscriptions.subscriptions,
            errors : RESPY_Background_Subscriptions.errors
        });
        return true;
    }
    if ((msg.from === 'content') && (msg.subject === 'extractRealtyObjectPhone') && msg.data) {
        RESPY_Background_Phone_Extractor.request(msg.data).then(response);
        return true;
    }
});

async function getRltbaseDomain(){
    return new Promise(function(resolve, reject){
        if(customRltbaseDomain === false){
            chrome.storage.sync.get([
                'respy_custom_server',
            ], function(response){
                resolve(response);
            });
        }else{
            resolve(customRltbaseDomain);
        }

    }).then((response) => {
        if(response.hasOwnProperty('respy_custom_server')){
            customRltbaseDomain = response;
            if(response.respy_custom_server !== ''){
                return response.respy_custom_server;
            }
        }
        return rltbaseDomain;
    });
}

async function extractPhoneSave(data){
    let port = '9081';
    let crm = data.crm;
    delete data.crm;
    let query = new URLSearchParams(data).toString();
    let url = `http://${rltbaseDomain}:${port}/phone_extractor_save?${query}`;
    let response = {};
    try {
        response = await fetch(url);
        response = await response.json();
    } catch (e) {
        response = {error : 'Ошибка запроса к базе собственников'};
    }
    return response;
}

async function extractPhone(data){
    let port = '9081';
    let crm = data.crm;
    delete data.crm;
    let query = new URLSearchParams(data).toString();
    let url = `http://${rltbaseDomain}:${port}/phone_extractor_commands?${query}`;
    let response = {};
    try {
        response = await fetch(url);
        response = await response.json();
    } catch (e) {
        response = {error : 'Ошибка запроса к базе собственников'};
    }
    return response;
}

function queryParams(data) {
    var array = [];

    for(var key in data) {
        array.push(encodeURIComponent(key) + "=" + encodeURIComponent(data[key]));
    }

    return array.join("&");
}

async function loadRltbase(data) {
    let port = '9081';
    let crm = data.crm;
    delete data.crm;
    let query = queryParams(data);
    let url = `http://${rltbaseDomain}:${port}/parse_object?${query}`;
    /*let url = `http://localhost:9081/parse_object?${query}`;*/

    let response = {};
    try {
        response = await fetch(url);
        response = await response.json();
    } catch (e) {
        console.log(e);
        response = {error : 'Ошибка запроса к базе собственников '+e.message};
    }

    if(response.hasOwnProperty('parser')) {
        response = {"wait" : "Создалось задание на парсинг объекта ожидайте", "parser" : response.parser};
    }

    if(response.hasOwnProperty('success') && response.success && response.hasOwnProperty('data')) {
        let crmData = response.data;
        if(crmData.hasOwnProperty('id') && crmData.hasOwnProperty('region')) {
            if(crm){
                response = await allowToCRM({
                    id : crmData.id,
                    region : crmData.region,
                    crm : crm
                });
            }else{
                response = {error : 'не верно указаны данные crm'};
            }

        } else {
            response = {error : 'Загрузка объекта в базу собственников прошла успешно, но данные неправильные'};
        }
    }
    return response;
}

async function allowToCRM(data) {
    let domain = data.crm;
    delete (data.crm);
    let query = new URLSearchParams(data).toString();
    let url = `${domain}/rltbase_connector/rltbase_parsed/allow_rltbase_object.json?${query}`;
    let response = {};
    try {
        response = await fetch(url);
        response = await response.json();
    } catch (e) {
        console.log(e);
        response = {error : 'Ошибка запроса к вашей CRM  попробуйте перейти по ссылке :'+url};
    }
    return response;
}

let RESPY_Background_Subscriptions = {
    subscriptions : null,
    errors : [],

    init : function () {
        this.getSubscriptions();
        setInterval( () => {
            this.getSubscriptions();
        },10000);
    },

    getSubscriptions : async function() {
        // Получаем информацию о подписках, отправляем её в popup и сетим badge
        this.errors = [];

        //Сначала получаем данные для ссылки из настроек.
        let settings = await new Promise(function(resolve, reject){
            chrome.storage.sync.get([
                'respy_subscription_domain',
                'respy_subscription_token',
                'respy_subscription_user_id'
            ], function(response){
                resolve(response);
            });
        });

        // Проверяем все ли настройки на месте
        if(!settings.respy_subscription_domain || !settings.respy_subscription_token || !settings.respy_subscription_user_id) {
            this.errors.push('Подписки не настроены');
            return;
        }

        let url = `http://${rltbaseDomain}:3031/subscriptions/domain/?token=${settings.respy_subscription_token}&domain=${settings.respy_subscription_domain}&user_id=${settings.respy_subscription_user_id}`;
        try {
            let response = await fetch(url);
            response = await response.json();
            let newObjectsCount = 0;
            if(!response.errors) {
                this.subscriptions = response.subscriptions;
                for (let key in response.subscriptions) {
                    let subscription = response.subscriptions[key];
                    newObjectsCount += subscription.count;
                }
                chrome.action.setBadgeText({text: newObjectsCount.toString()});
            } else {
                if(!Array.isArray(response.errors)) {
                    response.errors = [response.errors];
                }
                this.subscriptions = null;
                this.errors = response.errors;
            }
        } catch (e) {
            /*console.log(e);*/
        }
    }

};

let RESPY_Background_Notifications = {

    notifications : [],

    init : function () {
        // this.getNewNotifications();
        // setInterval( () => {
        //     this.getNewNotifications();
        // },10000);
    },

    getNewNotifications : async function() {
        // Получаем информацию о подписках, отправляем её в popup и сетим badge
        this.errors = [];

        //Сначала получаем данные для ссылки из настроек.
        let settings = await new Promise(function(resolve, reject){
            chrome.storage.sync.get([
                'respy_subscription_domain',
                'respy_subscription_user_id'
            ], function(response){
                resolve(response);
            });
        });

        // Проверяем все ли настройки на месте
        if(!settings.respy_subscription_domain || !settings.respy_subscription_user_id) {
            this.errors.push('Подписки не настроены');
            return;
        }

        let url = `http://${rltbaseDomain}:3031/object_subscriptions/new/?domain=${settings.respy_subscription_domain}&user_id=${settings.respy_subscription_user_id}&token=${tokenAllow}`;
        try {
            let response = await fetch(url);
            response = await response.json();
            if(RESPY_Background.isEmpty(response.errors)) {
                this.notifications = response.data;
                if(response.data.length > 0) {
                    chrome.action.setBadgeText({text: 'new'});
                }
            } else {
                if(!Array.isArray(response.errors)) {
                    response.errors = [response.errors];
                }
                this.errors = response.errors;
            }
        } catch (e) {
            console.log(e);
        }
    },

    getObjectChangeSubscriptions : async function() {
        // Получаем информацию о подписках, отправляем её в popup и сетим badge
        this.errors = [];

        //Сначала получаем данные для ссылки из настроек.
        let settings = await new Promise(function(resolve, reject){
            chrome.storage.sync.get([
                'respy_subscription_domain',
                'respy_subscription_user_id'
            ], function(response){
                resolve(response);
            });
        });

        // Проверяем все ли настройки на месте
        if(!settings.respy_subscription_domain || !settings.respy_subscription_user_id) {
            this.errors.push('Подписки не настроены');
            return;
        }

        let url = `http://${rltbaseDomain}:3031/object_subscriptions/?domain=${settings.respy_subscription_domain}&user_id=${settings.respy_subscription_user_id}&token=${tokenAllow}`;
        try {
            let response = await fetch(url);
            response = await response.json();
            return response;
        } catch (e) {
            /*console.log(e);*/
            return {
                'data': null,
                'errors': ['Запрос к серверу не выполнен. Попробуйте еще раз позднее']
            };
        }
    }

};

let RESPY_Background_Phone_Extractor = {
    init : function () {},
    request : async function (data) {
        if(!data.object || !data.object.source || !data.object.native_id) {
            return {
                success : false,
                error : 'Не хватает данных об объекте'
            };
        }

        let url = 'http://'+rltbaseDomain+':9081/phone_extractor_commands?url=';
        let response = {};
        try {
            let response = await fetch(url);
            response = await response.json();

        } catch (e) {
            return {success : false, error : 'Ошибка запроса к сервису'};
        }

        if(response.hasOwnProperty('success') && response.success && response.hasOwnProperty('data')) {
            let object = response.data;
            if(object.hasOwnProperty('commands')) {

                object.commands.forEach(function(item, i){
                    eval(item);
                });
                let clean_realtor_phone = $(object.resultSelector);
                return {
                    success : true,
                    message : 'Успешно получен номер телефона '+clean_realtor_phone,
                    response : object
                };
            } else {
                return {
                    success : false,
                    error : 'Не удалось получить телефон из объявления! ' +
                        'Возможно объявление не доступно для входящих звонков. ' +
                        'Перейдите в просмотр объявления, и получите необходимую информацию!'
                };
            }
        }


        /*if(!['youla'].includes(data.object.source)) {
            return {
                success : false,
                error : 'Для данной площадки пока нет возможности получить телефон. Следите за обновлениями.'
            };
        }

        let cookies = await this.getSourceCookie(data.object.source);

        let cookieError = this.getCookieError(cookies,data.object.source);
        if(cookieError) {
            return {success : false, error : cookieError};
        }

        data.cookie = this.getCookiesString(cookies);

        let url = `http://${rltbaseDomain}:3045/extract_phone`;
        let response = {};
        try {
            response = await $.post(url,data);
        } catch (e) {
            return {success : false, error : 'Ошибка запроса к сервису'};
        }

        if(response.hasOwnProperty('success') && response.success && response.hasOwnProperty('data')) {
            let object = response.data;
            if(object.hasOwnProperty('native_id') && object.hasOwnProperty('source') &&
                object.hasOwnProperty('clean_realtor_phone') && object['clean_realtor_phone'].length > 0
            ) {
                return {
                    success : true,
                    message : `Успешно получен номер телефона ${object.clean_realtor_phone}`,
                    response : object
                };
            } else {
                return {
                    success : false,
                    error : 'Не удалось получить телефон из объявления! ' +
                        'Возможно объявление не доступно для входящих звонков. ' +
                        'Перейдите в просмотр объявления, и получите необходимую информацию!'
                };
            }
        }
        */
        return  {
            success : false,
            error : "Необработанное исключение"
        }
    },

    getSourceCookie : async function (source) {
        switch (source) {
            case "youla" : {
                //Сначала получаем данные для ссылки из настроек.
                let cookies =  await new Promise(function(resolve, reject){
                    chrome.cookies.getAll({
                        domain : 'youla.ru'
                    },function (response) {
                        resolve(response);
                    });
                });
                return this.getCookies(cookies);
            }
            default : {
                return null;
            }
        }
    },

    getCookieError : function (cookies, source) {
        switch (source) {
            case "youla" : {
                if(!cookies || !cookies.youla_auth || !cookies.youla_auth_refresh || !cookies.sessid) {
                    return "Вы не авторизованы на площадке Юла. Пожалуйста, авторизуйтесь в соседней вкладке и повторите попытку";
                }
                return null;
            }
            default : {
                return null;
            }
        }
    },

    getCookies : function (cookies) {
        let c = {};
        cookies.forEach(function(cookie) {
            c[cookie.name] = cookie.value;
        });
        return c;
    },

    getCookiesString : function (cookies) {
        return Object
            .keys(cookies)
            .map(key => `${key}=${cookies[key]}`)
            .join(';');
    }
}

let RESPY_Background = {

    activeTab : null,
    tabs : {},

    init : function () {
        RESPY_Background_Subscriptions.init();
        RESPY_Background_Notifications.init();
        RESPY_Background_Phone_Extractor.init();
        this.listeners();
    },

    listeners : function () {
        // onActivated — происходит тогда, когда пользователь перешел на новую вкладку (по клику или по alt+tab).
        chrome.tabs.onActivated.addListener((info) => {
            this.onTabActivated(info);
        });

        // Будем слушать сообщения по порту
        chrome.runtime.onConnect.addListener((port) =>{
            port.onMessage.addListener(this.factory)
        });

        // onUpdated — происходит тогда, когда страница полностью (загрузился не только DOM, а и все картинки) загрузилась во вкладке.
        chrome.tabs.onUpdated.addListener((id, info, tab) => {
            this.onTabUpdated(id, info, tab);
        });

        chrome.tabs.onRemoved.addListener((tabId,removeInfo) => {
            RESPY_Background_Tabs.remove(tabId);
        });

    },

    onTabActivated: function(info) {
        let allowParse = this.getAllowParseData(info.tabId);
        if(!this.isEmpty(allowParse) && allowParse.allow) {
            chrome.action.setIcon({
                path : {
                    "16": "/img/allow_parse_icon16.png"
                }
            });
        } else {
            chrome.action.setIcon({
                path : {
                    "16": "/icon16.png"
                }
            });
        }
    },

    // Когда любая вкладка активируется, мы пытаемся её спарсить
    onTabUpdated : function(id, info, tab) {
        if (id && tab && info && info.status && (info.status.toLowerCase() === 'complete')){
            RESPY_Background_Tabs.push(tab);

            let port = chrome.tabs.connect(id);
            RESPY_Background_Tabs.tabs[id].port = port;

            RESPY_Background_Tabs.postMessage(id,{ method: 'allowParse', data: {tab_id : id} });

            RESPY_Background_Tabs.postMessage(id,{ method: 'getRealtyObjectData', data: {tab_id : id} });
        }
    },

    // Метод вызывается из content.js для сеттнга данных о парсинге этой страницы
    setAllowParseData : function(data) {
        RESPY_Background_Tabs.setData(data.tab_id,data, 'allowParse');
        this.onTabActivated( {tabId : data.tab_id})
    },

    // Метод вызывается из content.js для сеттнга данных о парсинге этой страницы
    setRealtyObjectData : function(data) {
        RESPY_Background_Tabs.setData(data.tab_id,data, 'RealtyObjectData');
    },

    // Метод вызывается из popup.js для получения данных с текущей вкладки
    getAllowParseData : function(tabId) {
        return RESPY_Background_Tabs.getData(tabId,'allowParse');
    },

    // Метод вызывается из popup.js для получения данных с текущей вкладки
    getRealtyObjectData : function(tabId) {
        return RESPY_Background_Tabs.getData(tabId,'RealtyObjectData');
    },

    extractRealtyObjectPhone : async function (data) {
        return RESPY_Background_Phone_Extractor.request(data);
    },

    factory : function (object) {
        if(object && object.method){
            if(object.data)
                RESPY_Background[object.method](object.data);
            else
                RESPY_Background[object.method]();
        }
    },

    isEmpty: function (obj) {

        // null and undefined are "empty"
        if (obj == null) return true;

        // Assume if it has a length property with a non-zero value
        // that that property is correct.
        if (obj.length > 0) return false;
        if (obj.length === 0) return true;

        // If it isn't an object at this point
        // it is empty, but it can't be anything *but* empty
        // Is it empty?  Depends on your application.
        if (typeof obj !== "object") return true;

        // Otherwise, does it have any properties of its own?
        // Note that this doesn't handle
        // toString and valueOf enumeration bugs in IE < 9
        for (var key in obj) {
            if (hasOwnProperty.call(obj, key)) return false;
        }

        return true;
    }
}

let RESPY_Background_Tabs = {
    tabs : {},

    push: function (tab) {
        if (tab.id && (tab.id !== 0)) {
            if (!this.tabs[tab.id])
                this.tabs[tab.id] = {tab_obj: tab};
        }
    },

    setData: function(tabId,data,dataKey) {
        if(this.tabs[tabId]){
            this.tabs[tabId][dataKey] = data;
        }
    },

    getData : function(tabId,dataKey) {
        if(this.tabs[tabId]){
            if(this.tabs[tabId][dataKey]) {
                return this.tabs[tabId][dataKey];
            }
        }
        return {};
    },

    remove : function(tabId) {
        delete this.tabs[tabId];
    },

    postMessage(tabId, message) {
        this.tabs[tabId].port.postMessage(message);
    }

};

RESPY_Background.init();

async function getTask() {
    let domainTask = 'rltbase.ru';
    getRltbaseDomain().then((result) => {
        domainTask = result;
        fetch("http://"+domainTask+":9081/v3/get").then((response) => {
            response.json().then((params) => {
                // console.log(params);
                if(params.hasOwnProperty('op') && params.op === 'no') {
                    // По  op - no ничего не делаем
                    return;
                }
                fetch(params.url, params).then((response) => {
                    params.status = response.status;
                    params.headers = {};
                    for (let pair of response.headers.entries()) {
                        params.headers[pair[0]] = pair[1];
                    }
                    response.text().then((html) => {
                        params.body = html;
                        let urlSendResult = "http://"+domainTask+":9081/v2/put";
                        fetch(urlSendResult, {method: 'post', body: JSON.stringify(params)});
                    });

                });

            });
        });
    });
}
getTask();
setInterval(getTask, 20000);
